@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: awcs-proto.c,v 1.35 2001/12/07 20:37:14 ela Exp $
+ * $Id: awcs-proto.c,v 1.37 2002/10/13 08:11:59 ela Exp $
  *
  */
 
@@ -84,6 +84,7 @@ svz_servertype_t awcs_server_definition =
   NULL,                  /* client info */
   NULL,                  /* server info */
   NULL,                  /* server timer */
+  NULL,                  /* server reset */
   NULL,                  /* handle request callback */
   &awcs_config,          /* the instance configuration */
   sizeof (awcs_config),  /* sizeof the instance configuration */
@@ -858,7 +859,7 @@ awcs_disconnected_socket (svz_socket_t *sock)
 
 /*
  * Gets called when the socket SOCK got kicked.  REASON is true when
- * a output buffer overflow occured and false, when it was an input
+ * a output buffer overflow occurred and false, when it was an input
  * buffer overflow or flooding.
  */
 int
