@@ -1,7 +1,7 @@
 /*
- * cfgfile.h - Configuration file definitions
+ * cfgfile.h - configuration file definitions
  *
- * Copyright (C) 2000 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2000, 2001 Stefan Jahn <stefan@lkcc.org>
  * Copyright (C) 2000 Raimund Jacob <raimi@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify it
@@ -19,18 +19,13 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: cfgfile.h,v 1.3 2000/09/08 07:45:16 ela Exp $
+ * $Id: cfgfile.h,v 1.6 2001/04/28 12:37:05 ela Exp $
  *
  */
 
 #ifndef __CFGFILE_H__
-#define __CFGFILE_H__
+#define __CFGFILE_H__ 1
 
-/*
- * How large should string variables be (bufferspace given to sizzle).
- */
-#define STRINGVARSIZE 1024
+void init_server_definitions (void);
 
-int load_config (char * cfgfilename, int argc, char **argv);
-
-#endif
+#endif /* not __CFGFILE_H__ */
