@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile-api.c,v 1.22 2002/02/15 12:16:07 ela Exp $
+ * $Id: guile-api.c,v 1.28 2002/07/17 18:34:18 ela Exp $
  *
  */
 
@@ -53,6 +53,12 @@
 #if HAVE_RPC_PMAP_PROT_H
 # include <rpc/pmap_prot.h>
 #endif
+#if HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+#if HAVE_IO_H
+# include <io.h>
+#endif
 
 #if GUILE_SOURCE
 # include <libguile/gh.h>
@@ -91,9 +97,9 @@ guile_resolve (char *host, unsigned long *addr)
    If @var{proto} equals @code{PROTO_ICMP} the @var{port} argument is
    ignored. Valid identifiers for @var{proto} are @code{PROTO_TCP},
    @code{PROTO_UDP} and @code{PROTO_ICMP}. The @var{host} argument must be 
-   either a string in dotted decimal form, a valid hostname or a exact number
+   either a string in dotted decimal form, a valid hostname or an exact number
    in host byte order. When giving a hostname this operation might be
-   blocking. The @var{port} argument must be a exact number in the range from 
+   blocking. The @var{port} argument must be an exact number in the range from 
    0 to 65535, also in host byte order. Returns a valid @code{#<svz-socket>} 
    or @code{#f} on failure. */
 #define FUNC_NAME "svz:sock:connect"
@@ -275,9 +281,66 @@ guile_sock_receive_buffer (SCM sock)
 }
 #undef FUNC_NAME
 
+/* Returns the current receive buffers size and fill status in bytes of 
+   the socket @var{sock} as a pair of exact numbers.  If the optional 
+   argument @var{size} is given the receive buffer will be set to the 
+   specified size in bytes. */
+#define FUNC_NAME "svz:sock:receive-buffer-size"
+static SCM
+guile_sock_receive_buffer_size (SCM sock, SCM size)
+{
+  svz_socket_t *xsock;
+  int len;
+
+  CHECK_SMOB_ARG (svz_socket, sock, SCM_ARG1, "svz-socket", xsock);
+  if (!SCM_UNBNDP (size))
+    {
+      SCM_ASSERT_TYPE (SCM_EXACTP (size), size, SCM_ARG2, FUNC_NAME, "exact");
+      len = SCM_NUM2INT (SCM_ARG2, size);
+      svz_sock_resize_buffers (xsock, xsock->send_buffer_size, len);
+    }
+  return scm_cons (scm_int2num (xsock->recv_buffer_size), 
+		   scm_int2num (xsock->recv_buffer_fill));
+}
+#undef FUNC_NAME
+
+/* Return the send buffer of the socket @var{sock} as a binary smob. */
+#define FUNC_NAME "svz:sock:send-buffer"
+static SCM
+guile_sock_send_buffer (SCM sock)
+{
+  svz_socket_t *xsock;
+  CHECK_SMOB_ARG (svz_socket, sock, SCM_ARG1, "svz-socket", xsock);
+  return guile_data_to_bin (xsock->send_buffer, xsock->send_buffer_fill);
+}
+#undef FUNC_NAME
+
+/* This procedure returns the current send buffer size and fill status in 
+   bytes of the socket @var{sock} as a pair of exact numbers.  If the 
+   optional argument @var{size} is given the send buffer will be set to 
+   the specified size in bytes. */
+#define FUNC_NAME "svz:sock:send-buffer-size"
+static SCM
+guile_sock_send_buffer_size (SCM sock, SCM size)
+{
+  svz_socket_t *xsock;
+  int len;
+
+  CHECK_SMOB_ARG (svz_socket, sock, SCM_ARG1, "svz-socket", xsock);
+  if (!SCM_UNBNDP (size))
+    {
+      SCM_ASSERT_TYPE (SCM_EXACTP (size), size, SCM_ARG2, FUNC_NAME, "exact");
+      len = SCM_NUM2INT (SCM_ARG2, size);
+      svz_sock_resize_buffers (xsock, len, xsock->recv_buffer_size);
+    }
+  return scm_cons (scm_int2num (xsock->send_buffer_size), 
+		   scm_int2num (xsock->send_buffer_fill));
+}
+#undef FUNC_NAME
+
 /* Dequeue @var{length} bytes from the receive buffer of the socket 
    @var{sock} which must be a valid @code{#<svz-socket>}. If the user omits
-   the optional @var{length} argument all of the data in the receive buffer
+   the optional @var{length} argument, all of the data in the receive buffer
    gets dequeued. Returns the number of bytes actually shuffled away. */
 #define FUNC_NAME "svz:sock:receive-buffer-reduce"
 static SCM
@@ -308,7 +371,7 @@ guile_sock_receive_buffer_reduce (SCM sock, SCM length)
 
 /* This procedure returns the current remote address as a pair like 
    @code{(host . port)} with both entries in network byte order. If you pass
-   the optional argument @var{address} you can set the remote address of 
+   the optional argument @var{address}, you can set the remote address of 
    the socket @var{sock}. */
 #define FUNC_NAME "svz:sock:remote-address"
 static SCM
@@ -336,7 +399,7 @@ guile_sock_remote_address (SCM sock, SCM address)
 
 /* This procedure returns the current local address as a pair like 
    @code{(host . port)} with both entries in network byte order. If you pass
-   the optional argument @var{address} you can set the local address of 
+   the optional argument @var{address}, you can set the local address of 
    the socket @var{sock}. */
 #define FUNC_NAME "svz:sock:local-address"
 static SCM
@@ -362,7 +425,7 @@ guile_sock_local_address (SCM sock, SCM address)
 }
 #undef FUNC_NAME
 
-/* Return the given sockets @var{sock} parent and optionally set it to the
+/* Return the given socket's @var{sock} parent and optionally set it to the
    socket @var{parent}. The procedure returns either a valid 
    @code{#<svz-socket>} object or an empty list. */
 #define FUNC_NAME "svz:sock:parent"
@@ -384,7 +447,7 @@ guile_sock_parent (SCM sock, SCM parent)
 }
 #undef FUNC_NAME
 
-/* Return the given sockets @var{sock} referrer and optionally set it to the
+/* Return the given socket's @var{sock} referrer and optionally set it to the
    socket @var{referrer}. The procedure returns either a valid 
    @code{#<svz-socket>} or an empty list. */
 #define FUNC_NAME "svz:sock:referrer"
@@ -410,8 +473,8 @@ guile_sock_referrer (SCM sock, SCM referrer)
    given argument @var{sock}. The optional argument @var{server} can be used
    to redefine this association and must be a valid @code{#<svz-server>}
    object. For a usual socket callback like @code{connect-socket} or
-   @code{handle-request} the association is already in place. But for sockets
-   created by @code{(svz:sock:connect)} you can use it in order to make the
+   @code{handle-request}, the association is already in place. But for sockets
+   created by @code{(svz:sock:connect)}, you can use it in order to make the
    returned socket object part of a server. */
 #define FUNC_NAME "svz:sock:server"
 static SCM
@@ -433,8 +496,23 @@ guile_sock_server (SCM sock, SCM server)
 }
 #undef FUNC_NAME
 
+/* Returns one of the @code{PROTO_TCP}, @code{PROTO_UDP}, @code{PROTO_ICMP}, 
+   @code{PROTO_RAW} or @code{PROTO_PIPE} constants indicating the type of 
+   the socket structure @var{sock}. If there is no protocol information 
+   available the procedure returns @code{#f}. */
+#define FUNC_NAME "svz:sock:protocol"
+static SCM
+guile_sock_protocol (SCM sock)
+{
+  svz_socket_t *xsock;
+
+  CHECK_SMOB_ARG (svz_socket, sock, SCM_ARG1, "svz-socket", xsock);
+  return scm_int2num (xsock->proto);
+}
+#undef FUNC_NAME
+
 /* This procedure schedules the socket @var{sock} for shutdown after all data
-   within the send buffer queue has been send. The user should issue this
+   within the send buffer queue has been sent. The user should issue this
    procedure call right *before* the last call to @code{(svz:sock:print)}. */
 #define FUNC_NAME "svz:sock:final-print"
 static SCM
@@ -503,7 +581,7 @@ guile_server_p (SCM server)
 /* Set the @code{disconnected-socket} member of the socket structure 
    @var{sock} to the Guile procedure @var{proc}. The given callback
    runs whenever the socket is lost for some external reason. The procedure 
-   returns the previously set handler if there is any. */
+   returns the previously set handler if there is one. */
 #define FUNC_NAME "svz:sock:disconnected"
 MAKE_SOCK_CALLBACK (disconnected_socket, "disconnected")
 #undef FUNC_NAME
@@ -516,6 +594,23 @@ MAKE_SOCK_CALLBACK (disconnected_socket, "disconnected")
 MAKE_SOCK_CALLBACK (kicked_socket, "kicked")
 #undef FUNC_NAME
 
+/* This procedure sets the @code{trigger-condition} callback for the socket
+   structure @var{sock} to the Guile procedure @var{proc}.  It returns the
+   previously set procedure if available.  The callback is run once every
+   server loop indicating whether the @code{trigger} callback should be
+   run or not. */
+#define FUNC_NAME "svz:sock:trigger-condition"
+MAKE_SOCK_CALLBACK (trigger_cond, "trigger-condition")
+#undef FUNC_NAME
+
+/* Sets the @code{trigger} callback of the socket structure @var{sock} to
+   the Guile procedure @var{proc} and returns any previously set procedure.
+   The callback is run when the @code{trigger-condition} callback returned
+   @code{#t}. */
+#define FUNC_NAME "svz:sock:trigger"
+MAKE_SOCK_CALLBACK (trigger_func, "trigger")
+#undef FUNC_NAME
+
 /* This procedure sets the @code{idle} callback of the socket structure
    @var{sock} to the Guile procedure @var{proc}. It returns any previously
    set procedure. The callback is run by the periodic task scheduler when the
@@ -527,9 +622,18 @@ MAKE_SOCK_CALLBACK (kicked_socket, "kicked")
 MAKE_SOCK_CALLBACK (idle_func, "idle")
 #undef FUNC_NAME
 
-/* This functions returns the socket structure's @var{sock} current value 
-   of the @code{idle-counter}. If the optional argument @var{counter} is
-   given the function sets the @code{idle-counter}. Please have a look at the
+/* With this procedure you can setup the @code{check-oob-request} callback
+   of the given socket structure @var{sock}.  The previous callback is 
+   replaced by the @var{proc} procedure and will be returned if there was
+   set any before.  The callback is run whenever urgent data (out-of-band)
+   has been detected on the socket. */
+#define FUNC_NAME "svz:sock:check-oob-request"
+MAKE_SOCK_CALLBACK (check_request_oob, "check-oob-request")
+#undef FUNC_NAME
+
+/* This functions returns the socket structure @var{sock}'s current 
+   @code{idle-counter} value. If the optional argument @var{counter} is
+   given, the function sets the @code{idle-counter}. Please have a look at the
    @code{(svz:sock:idle)} procedure for the exact meaning of this value. */
 #define FUNC_NAME "svz:sock:idle-counter"
 static SCM
@@ -551,7 +655,7 @@ guile_sock_idle_counter (SCM sock, SCM counter)
 #undef FUNC_NAME
 
 /* Returns a list of listening @code{#<svz-socket>} smobs to which the 
-   given server instance @var{server} is currently bound to or an empty list
+   given server instance @var{server} is currently bound, or an empty list
    if there is no such binding yet. */
 #define FUNC_NAME "svz:server:listeners"
 static SCM
@@ -679,7 +783,7 @@ scm_setrpc (SCM stayopen)
 /* This procedure returns a list of the current RPC program-to-port mappings
    on the host located at IP address @var{address}. When you leave this 
    argument it defaults to the local machine's IP address. This routine 
-   can return an empty list either indicating there is no such list 
+   can return an empty list indicating either there is no such list 
    available or an error occurred while fetching the list. */
 #define FUNC_NAME "portmap-list"
 SCM
@@ -736,7 +840,7 @@ scm_portmap_list (SCM address)
    between the triple [@var{prognum},@var{versnum},@var{protocol}] and 
    @var{port} on the machine's portmap service. The value of @var{protocol}
    is most likely @code{IPPROTO_UDP} or @code{IPPROTO_TCP}.
-   If the user omits @var{protocol} and @var{port} the procedure destroys
+   If the user omits @var{protocol} and @var{port}, the procedure destroys
    all mapping between the triple [@var{prognum},@var{versnum},*] and ports
    on the machine's portmap service. */
 #define FUNC_NAME "portmap"
@@ -801,12 +905,12 @@ guile_coserver_callback (char *res, SCM callback, SCM arg)
 }
 
 /* This procedure enqueues the @var{host} string argument into the internal
-   dns coserver queue. When the coserver responds the Guile procedure
+   DNS coserver queue. When the coserver responds, the Guile procedure
    @var{callback} is run as @code{(callback addr arg)}. The @var{addr} 
    argument passed to the callback is a string representing the appropriate 
    IP address for the given hostname @var{host}. If you omit the optional 
    argument @var{arg} it is run as @code{(callback addr)} only. The @var{arg}
-   argument may  be necessary if you need to have the callback procedure 
+   argument may be necessary if you need to have the callback procedure 
    in a certain context. */ 
 #define FUNC_NAME "svz:coserver:dns"
 SCM
@@ -837,9 +941,9 @@ guile_coserver_dns (SCM host, SCM callback, SCM arg)
 #undef FUNC_NAME
 
 /* This Guile procedure enqueues the given @var{addr} argument which must be
-   an IP address in network byte order into the internal reverse dns coserver
-   queue. When the coserver responds the Guile procedure @var{callback} is
-   run as @code{(callback host arg)} whereas @var{host} is the hostname of the
+   an IP address in network byte order into the internal reverse DNS coserver
+   queue. When the coserver responds, the Guile procedure @var{callback} is
+   run as @code{(callback host arg)} where @var{host} is the hostname of the
    requested IP address @var{addr}. The last argument @var{arg} is 
    optional. */
 #define FUNC_NAME "svz:coserver:reverse-dns"
@@ -869,8 +973,8 @@ guile_coserver_rdns (SCM addr, SCM callback, SCM arg)
 #undef FUNC_NAME
 
 /* This procedure enqueues the given @code{#<svz-socket>} @var{sock} into the
-   internal ident coserver queue. When the coserver responds it runs the
-   Guile procedure @var{callback} as @code{(callback user arg)} whereas 
+   internal ident coserver queue. When the coserver responds, it runs the
+   Guile procedure @var{callback} as @code{(callback user arg)} where
    @var{user} is the corresponding username for the client connection 
    @var{sock}. The @var{arg} argument is optional. */
 #define FUNC_NAME "svz:coserver:ident"
@@ -896,7 +1000,7 @@ guile_coserver_ident (SCM sock, SCM callback, SCM arg)
 }
 #undef FUNC_NAME
 
-/* The given argument @var{ident} must be a pair of numbers whereas the
+/* The given argument @var{ident} must be a pair of numbers where the
    car is a @code{#<svz-socket>}'s identification number and the cdr the 
    version number. The procedure returns either the identified 
    @code{#<svz-socket>} or @code{#f} if the given combination is not 
@@ -935,6 +1039,90 @@ guile_sock_ident (SCM sock)
 }
 #undef FUNC_NAME
 
+/* This procedure returns either a binary smob containing a data block read
+   from the open input port @var{port} with a maximum number of @var{size}
+   bytes or the end-of-file object if the underlying ports end has been 
+   reached.  The size of the returned binary smob may be less than the 
+   requested size @var{size} if it exceed the current size of the given port
+   @var{port}.  The procedure throws an exception if an error occurred while
+   reading from the port. */
+#define FUNC_NAME "svz:read-file"
+static SCM
+guile_read_file (SCM port, SCM size)
+{
+  int fdes, len, ret;
+  void *data;
+
+  /* Check argument list. */
+  SCM_ASSERT_TYPE (SCM_NIMP (port) && SCM_FPORTP (port) && 
+		   SCM_OPINFPORTP (port),
+                   port, SCM_ARG1, FUNC_NAME, "open input port");
+  SCM_ASSERT_TYPE (SCM_EXACTP (size), size, SCM_ARG2, FUNC_NAME, "exact");
+
+  /* Get underlying file descriptor. */
+  fdes = SCM_FPORT_FDES (port);
+
+  if ((len = SCM_NUM2INT (SCM_ARG1, size)) <= 0)
+    SCM_OUT_OF_RANGE (SCM_ARG2, size);
+
+  /* Allocate necessary data. */
+  data = (unsigned char *) scm_must_malloc (len, "svz-binary-data");
+
+  /* Read from file descriptor and evaluate return value. */
+  if ((ret = read (fdes, data, len)) < 0)
+    {
+      scm_must_free (data);
+      scm_syserror_msg (FUNC_NAME, "~A: read ~A ~A",
+			scm_listify (scm_makfrom0str (strerror (errno)),
+				     scm_int2num (fdes), size, SCM_UNDEFINED),
+			errno);
+    }
+  else if (ret == 0)
+    {
+      scm_must_free (data);
+      return SCM_EOF_VAL;
+    }
+  else if (ret != len)
+    {
+      data = (unsigned char *) 
+	scm_must_realloc (data, len, ret, "svz-binary-data");
+    }
+
+  /* Finally return binary smob. */
+  return guile_garbage_to_bin (data, ret);
+}
+#undef FUNC_NAME
+
+/* This procedure expects a TCP @code{#<svz-socket>} in @var{sock} and an 
+   exact number or single character in @var{oob}.  The byte in @var{oob} 
+   is sent as urgent (out-of-band) data through the underlying TCP stream.
+   The procedure returns @code{#t} on successful completion and otherwise
+   (either it failed to send the byte or the passed socket is not a TCP 
+   socket) @code{#f}. */
+#define FUNC_NAME "svz:sock:send-oob"
+static SCM
+guile_sock_send_oob (SCM sock, SCM oob)
+{
+  svz_socket_t *xsock;
+  int ret = -1;
+
+  /* Check the arguments. */
+  CHECK_SMOB_ARG (svz_socket, sock, SCM_ARG1, "svz-socket", xsock);
+  SCM_ASSERT_TYPE (SCM_EXACTP (oob) || SCM_CHARP (oob), 
+		   oob, SCM_ARG2, FUNC_NAME, "char or exact");
+
+  /* Send the oob byte through TCP sockets only. */
+  if (xsock->proto & PROTO_TCP)
+    {
+      xsock->oob = (unsigned char)
+	(SCM_CHARP (oob) ? SCM_CHAR (oob) :
+	 (unsigned char) SCM_NUM2INT (SCM_ARG2, oob));
+      ret = svz_tcp_send_oob (xsock);
+    }
+  return ((ret < 0) ? SCM_BOOL_F : SCM_BOOL_T);
+}
+#undef FUNC_NAME
+
 /* Initialize the API function calls supported by Guile. */
 void
 guile_api_init (void)
@@ -957,6 +1145,8 @@ guile_api_init (void)
   scm_c_define ("PROTO_TCP", scm_int2num (PROTO_TCP));
   scm_c_define ("PROTO_UDP", scm_int2num (PROTO_UDP));
   scm_c_define ("PROTO_ICMP", scm_int2num (PROTO_ICMP));
+  scm_c_define ("PROTO_RAW", scm_int2num (PROTO_RAW));
+  scm_c_define ("PROTO_PIPE", scm_int2num (PROTO_PIPE));
   scm_c_define ("KICK_FLOOD", scm_int2num (0));
   scm_c_define ("KICK_QUEUE", scm_int2num (1));
   scm_c_define_gsubr ("svz:sock:connect", 2, 1, 0, guile_sock_connect);
@@ -969,6 +1159,7 @@ guile_api_init (void)
   scm_c_define_gsubr ("svz:sock:parent", 1, 1, 0, guile_sock_parent);
   scm_c_define_gsubr ("svz:sock:referrer", 1, 1, 0, guile_sock_referrer);
   scm_c_define_gsubr ("svz:sock:server", 1, 1, 0, guile_sock_server);
+  scm_c_define_gsubr ("svz:sock:protocol", 1, 0, 0, guile_sock_protocol);
   scm_c_define_gsubr ("svz:sock:final-print", 1, 0, 0, guile_sock_final_print);
   scm_c_define_gsubr ("svz:sock:no-delay", 1, 1, 0, guile_sock_no_delay);
   scm_c_define_gsubr ("svz:sock:ident", 1, 0, 0, guile_sock_ident);
@@ -976,8 +1167,13 @@ guile_api_init (void)
   scm_c_define_gsubr ("svz:sock?", 1, 0, 0, guile_sock_p);
   scm_c_define_gsubr ("svz:server?", 1, 0, 0, guile_server_p);
   scm_c_define_gsubr ("svz:server:listeners", 1, 0, 0, guile_server_listeners);
-  scm_c_define_gsubr ("svz:sock:receive-buffer", 
+  scm_c_define_gsubr ("svz:sock:send-buffer", 1, 0, 0, guile_sock_send_buffer);
+  scm_c_define_gsubr ("svz:sock:send-buffer-size",
+		      1, 1, 0, guile_sock_send_buffer_size);
+  scm_c_define_gsubr ("svz:sock:receive-buffer",
 		      1, 0, 0, guile_sock_receive_buffer);
+  scm_c_define_gsubr ("svz:sock:receive-buffer-size",
+		      1, 1, 0, guile_sock_receive_buffer_size);
   scm_c_define_gsubr ("svz:sock:receive-buffer-reduce",
 		      1, 1, 0, guile_sock_receive_buffer_reduce);
   scm_c_define_gsubr ("svz:sock:remote-address",
@@ -992,10 +1188,15 @@ guile_api_init (void)
 		      2, 1, 0, guile_coserver_rdns);
   scm_c_define_gsubr ("svz:coserver:ident",
 		      2, 1, 0, guile_coserver_ident);
+  scm_c_define_gsubr ("svz:read-file", 2, 0, 0, guile_read_file);
+  scm_c_define_gsubr ("svz:sock:send-oob", 2, 0, 0, guile_sock_send_oob);
 
+  DEFINE_SOCK_CALLBACK ("svz:sock:trigger-condition",trigger_cond);
+  DEFINE_SOCK_CALLBACK ("svz:sock:trigger",trigger_func);
   DEFINE_SOCK_CALLBACK ("svz:sock:disconnected",disconnected_socket);
   DEFINE_SOCK_CALLBACK ("svz:sock:kicked",kicked_socket);
   DEFINE_SOCK_CALLBACK ("svz:sock:idle",idle_func);
+  DEFINE_SOCK_CALLBACK ("svz:sock:check-oob-request",check_request_oob);
 }
 
 /* Finalize the API functions. */
