@@ -1,7 +1,7 @@
 /*
  * guile-api.h - compatibility and miscellaneous guile functionality
  *
- * Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,13 +18,23 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile-api.h,v 1.7 2002/02/12 11:45:55 ela Exp $
+ * $Id: guile-api.h,v 1.11 2002/07/15 07:45:04 ela Exp $
  *
  */
 
 #ifndef __GUILE_API_H__
 #define __GUILE_API_H__ 1
 
+/* Define this macro if Guile 1.5.x or better is in use. */
+#if defined (SCM_MINOR_VERSION) && (SCM_MINOR_VERSION >= 5) && \
+    defined (SCM_MAJOR_VERSION) && (SCM_MAJOR_VERSION >= 1)
+#define SCM_VERSION_15X 1
+#endif
+
+#ifndef SCM_VERSION_15X
+#define scm_t_bits long
+#endif
+
 /* Some definitions for backward compatibility with Guile 1.3.4 */
 #ifndef SCM_ASSERT_TYPE
 #define SCM_ASSERT_TYPE(_cond, _arg, _pos, _subr, _msg) \
@@ -35,6 +45,17 @@
     scm_out_of_range (_subr, _bad)
 #endif /* not SCM_ASSERT_TYPE */
 
+/* Redefinition of the string and symbol predicates because they segfault
+   for Guile 1.3.4 and prior version when passing immediate values. */
+#ifdef SCM_STRINGP
+#undef SCM_STRINGP
+#endif
+#define SCM_STRINGP(obj) SCM_NFALSEP (scm_string_p (obj))
+#ifdef SCM_SYMBOLP
+#undef SCM_SYMBOLP
+#endif
+#define SCM_SYMBOLP(obj) SCM_NFALSEP (scm_symbol_p (obj))
+
 /* Compatibility definitions for various Guile versions.  These definitions
    are mainly due to the fact that the gh interface is deprecated in newer
    versions. */
@@ -80,7 +101,7 @@
 #ifndef SCM_MAKE_CHAR
 #define SCM_MAKE_CHAR(x) SCM_MAKICHR (x)
 #endif
-#ifndef scm_num2int
+#ifndef SCM_NUM2INT
 #define SCM_NUM2INT(pos, obj) gh_scm2int (obj)
 #endif
 #ifndef SCM_NUM2LONG
@@ -89,27 +110,30 @@
 #ifndef SCM_NUM2ULONG
 #define SCM_NUM2ULONG(pos, obj) scm_num2ulong (obj, (char *) (pos), FUNC_NAME)
 #endif
-#ifndef scm_int2num
+#ifndef SCM_VERSION_15X
 #define scm_int2num(x) scm_long2num ((long) (x))
 #endif
-#ifndef scm_mem2string
+#ifndef SCM_VERSION_15X
 #define scm_mem2string(str, len) gh_str2scm (str, len)
 #endif
-#ifndef scm_c_define
+#ifndef SCM_VERSION_15X
+#define scm_primitive_eval_x(expr) scm_eval_x (expr)
+#endif
+#ifndef SCM_VERSION_15X
 #define scm_c_define(name, val) gh_define (name, val)
 #endif
 #ifndef scm_c_free
 #define scm_c_free(p) scm_must_free (p)
 #endif
-#ifndef scm_c_define_gsubr
+#ifndef SCM_VERSION_15X
 #define scm_c_define_gsubr(name, req, opt, rst, fcn) \
     gh_new_procedure (name, fcn, req, opt, rst)
 #endif
-#ifndef scm_c_primitive_load
+#ifndef SCM_VERSION_15X
 #define scm_c_primitive_load(file) \
     scm_primitive_load (scm_makfrom0str (file))
 #endif
-#ifndef scm_current_module_lookup_closure
+#ifndef SCM_VERSION_15X
 #define guile_lookup(var, name) (var) = gh_lookup (name)
 #else
 #define guile_lookup(var, name) do {                                        \
@@ -117,13 +141,13 @@
 	  	         scm_current_module_lookup_closure (), SCM_BOOL_F); \
     if (SCM_FALSEP (var)) (var) = SCM_UNDEFINED; } while (0)
 #endif
-#ifndef scm_gc_protect_object
+#ifndef SCM_VERSION_15X
 #define scm_gc_protect_object(obj) scm_protect_object (obj)
 #endif
-#ifndef scm_gc_unprotect_object
+#ifndef SCM_VERSION_15X
 #define scm_gc_unprotect_object(obj) scm_unprotect_object (obj)
 #endif
-#ifndef scm_c_make_vector
+#ifndef SCM_VERSION_15X
 #define scm_c_make_vector(k, fill) scm_make_vector (scm_int2num (k), fill)
 #endif
 #ifndef SCM_OUT_OF_RANGE
@@ -131,8 +155,8 @@
     scm_out_of_range_pos (FUNC_NAME, arg, SCM_MAKINUM (pos))
 #endif
 
-/* Return an integer. If the given Guile cell @var{obj} is not such an 
-   integer the routine return the default value @var{def}. */
+/* Return an integer. If the given Guile cell @var{obj} is not an 
+   integer, the routine returns the default value @var{def}. */
 #define guile_integer(pos, obj, def) \
     ((SCM_EXACTP (obj)) ? (SCM_NUM2INT (pos, obj)) : (def))
 
@@ -165,5 +189,8 @@
 #define gh_scm2chars(obj, lenp) guile_to_string (obj)
 #define HAVE_OLD_SMOBS 1
 #endif
+#ifndef SCM_FPORT_FDES
+#define SCM_FPORT_FDES(port) fileno ((FILE *) SCM_STREAM (port))
+#endif
 
 #endif /* not __GUILE_API_H__ */
