@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile-bin.c,v 1.19 2002/02/15 12:16:07 ela Exp $
+ * $Id: guile-bin.c,v 1.27 2002/07/27 13:32:14 ela Exp $
  *
  */
 
@@ -55,7 +55,7 @@ typedef struct guile_bin
 guile_bin_t;
 
 /* The smob tag. */
-static long guile_bin_tag = 0;
+static scm_t_bits guile_bin_tag = 0;
 
 /* Useful defines for accessing the binary smob. */
 #define GET_BIN_SMOB(binary) \
@@ -147,9 +147,18 @@ guile_string_to_bin (SCM string)
 
   bin = MAKE_BIN_SMOB ();
   bin->size = SCM_NUM2INT (SCM_ARG1, scm_string_length (string));
-  bin->data = (unsigned char *) scm_must_malloc (bin->size, "svz-binary-data");
-  memcpy (bin->data, SCM_STRING_CHARS (string), bin->size);
-  bin->garbage = 1;
+  if (bin->size > 0)
+    {
+      bin->data = (unsigned char *) 
+	scm_must_malloc (bin->size, "svz-binary-data");
+      memcpy (bin->data, SCM_STRING_CHARS (string), bin->size);
+      bin->garbage = 1;
+    }
+  else
+    {
+      bin->data = NULL;
+      bin->garbage = 0;
+    }
 
   SCM_RETURN_NEWSMOB (guile_bin_tag, bin);
 }
@@ -169,7 +178,7 @@ guile_bin_to_string (SCM binary)
 #undef FUNC_NAME
 
 /* This routine searches through the binary smob @var{binary} for the cell
-   @var{needle}. The latter argument can be either a exact number, character,
+   @var{needle}. The latter argument can be either an exact number, character,
    string or another binary smob. It returns @code{#f} if the needle could 
    not be found and a positive number indicates the position of the first 
    occurrence of @var{needle} in the binary smob @var{binary}. */
@@ -200,6 +209,11 @@ guile_bin_search (SCM binary, SCM needle)
       start = bin->data;
       end = start + bin->size - len;
 
+      /* Return #f if searching in empty data sets. */
+      if (len == 0 || p == NULL || start == NULL)
+	return ret;
+
+      /* Iterate the data. */
       while (start <= end)
 	{
 	  if (*start == *p && memcmp (start, p, len) == 0)
@@ -217,8 +231,9 @@ guile_bin_search (SCM binary, SCM needle)
       unsigned char c;
       unsigned char *p, *end;
 
-      c = SCM_CHARP (needle) ? 
-	SCM_CHAR (needle) : (unsigned char) SCM_NUM2INT (SCM_ARG2, needle);
+      c = (unsigned char)
+	(SCM_CHARP (needle) ? SCM_CHAR (needle) : 
+	 (unsigned char) SCM_NUM2INT (SCM_ARG2, needle));
       p = bin->data;
       end = p + bin->size;
 
@@ -236,8 +251,64 @@ guile_bin_search (SCM binary, SCM needle)
 }
 #undef FUNC_NAME
 
+/* Performs an in place reversal of the given binary smob @var{binary}
+   and returns it. */
+#define FUNC_NAME "binary-reverse!"
+SCM
+guile_bin_reverse_x (SCM binary)
+{
+  guile_bin_t *bin;
+  int first, last;
+  unsigned char b;
+
+  CHECK_BIN_SMOB_ARG (binary, SCM_ARG1, bin);
+
+  for (first = 0, last = bin->size - 1; first < last; first++, last--)
+    {
+      b = bin->data[first];
+      bin->data[first] = bin->data[last];
+      bin->data[last] = b;
+    }
+  return binary;
+}
+#undef FUNC_NAME
+
+/* Returns a new binary smob with the reverse byte order of the given
+   binary smob @var{binary}. */
+#define FUNC_NAME "binary-reverse"
+SCM
+guile_bin_reverse (SCM binary)
+{
+  guile_bin_t *bin, *reverse;
+  int first, last;
+
+  CHECK_BIN_SMOB_ARG (binary, SCM_ARG1, bin);
+  reverse = MAKE_BIN_SMOB ();
+  reverse->size = bin->size;
+
+  /* Return empty smob if necessary. */
+  if (reverse->size == 0)
+    {
+      reverse->garbage = 0;
+      reverse->data = NULL;
+      SCM_RETURN_NEWSMOB (guile_bin_tag, reverse);
+    }
+
+  /* Reserve some memory for the new smob. */
+  reverse->data = (unsigned char *) 
+    scm_must_malloc (reverse->size, "svz-binary-data");
+  reverse->garbage = 1;
+
+  /* Apply reverse byte order to the new smob. */
+  for (first = 0, last = reverse->size - 1; first < reverse->size; )
+    reverse->data[first++] = bin->data[last--];
+
+  SCM_RETURN_NEWSMOB (guile_bin_tag, reverse);
+}
+#undef FUNC_NAME
+
 /* Set the byte at position @var{index} of the binary smob @var{binary} to
-   the value given in @var{value} which can be either a character or a 
+   the value given in @var{value} which can be either a character or an
    exact number. */
 #define FUNC_NAME "binary-set!"
 SCM
@@ -256,7 +327,9 @@ guile_bin_set_x (SCM binary, SCM index, SCM value)
   if (idx < 0 || idx >= bin->size)
     SCM_OUT_OF_RANGE (SCM_ARG2, index);
 
-  bin->data[idx] = SCM_CHAR (value);
+  bin->data[idx] = (unsigned char)
+    (SCM_CHARP (value) ? SCM_CHAR (value) : 
+     (unsigned char) SCM_NUM2INT (SCM_ARG3, value));
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -282,7 +355,7 @@ guile_bin_ref (SCM binary, SCM index)
 }
 #undef FUNC_NAME
 
-/* Return the size of the binary smob @var{binary}. */
+/* Return the size in bytes of the binary smob @var{binary}. */
 #define FUNC_NAME "binary-length"
 SCM
 guile_bin_length (SCM binary)
@@ -303,9 +376,10 @@ SCM
 guile_bin_concat_x (SCM binary, SCM append)
 {
   guile_bin_t *bin, *concat = NULL;
-  int len;
+  int len, equal;
   unsigned char *p;
   
+  /* Check arguments first. */
   CHECK_BIN_SMOB_ARG (binary, SCM_ARG1, bin);
   SCM_ASSERT (SCM_STRINGP (append) || CHECK_BIN_SMOB (append),
 	      append, SCM_ARG2, FUNC_NAME);
@@ -315,6 +389,11 @@ guile_bin_concat_x (SCM binary, SCM append)
   len = concat ? 
     concat->size : SCM_NUM2INT (SCM_ARG2, scm_string_length (append));
   p = concat ? concat->data : SCM_STRING_UCHARS (append);
+  equal = (p == bin->data) ? 1 : 0;
+
+  /* Return here if there is nothing to concatenate. */
+  if (len <= 0)
+    return binary;
 
   if (bin->garbage)
     {
@@ -330,6 +409,9 @@ guile_bin_concat_x (SCM binary, SCM append)
       memcpy (bin->data, odata, bin->size);
     }
 
+  /* Reapply concatenation pointer if identical binaries have been passed. */
+  p = equal ? bin->data : p;
+
   memcpy (bin->data + bin->size, p, len);
   bin->size += len;
   bin->garbage = 1;
@@ -360,9 +442,9 @@ guile_bin_subset (SCM binary, SCM start, SCM end)
     to = bin->size - 1;
 
   /* Check the ranges of both indices. */
-  if (from < 0 || from >= bin->size || from >= end)
+  if (from < 0 || from >= bin->size)
     SCM_OUT_OF_RANGE (SCM_ARG2, start);
-  if (to < 0 || to >= bin->size || to <= from)
+  if (to < 0 || to >= bin->size || to < from)
     SCM_OUT_OF_RANGE (SCM_ARG3, end);
 
   ret = MAKE_BIN_SMOB ();
@@ -392,7 +474,8 @@ guile_bin_to_list (SCM binary)
 #undef FUNC_NAME
 
 /* Convert the scheme list @var{list} into a binary smob. Each of the 
-   elements of @var{list} is checked for validity. */
+   elements of @var{list} is checked for validity.  The elements can be
+   either exact numbers in a byte's range or characters. */
 #define FUNC_NAME "list->binary"
 SCM
 guile_list_to_bin (SCM list)
@@ -400,6 +483,7 @@ guile_list_to_bin (SCM list)
   guile_bin_t *bin;
   unsigned char *p;
   int value;
+  SCM val;
 
   SCM_ASSERT_TYPE (SCM_LISTP (list), list, SCM_ARG1, FUNC_NAME, "list");
   bin = MAKE_BIN_SMOB ();
@@ -421,12 +505,22 @@ guile_list_to_bin (SCM list)
   /* Iterate over the list and build up binary smob. */
   while (SCM_PAIRP (list))
     {
-      if (!SCM_EXACTP (SCM_CAR (list)))
-	scm_out_of_range (FUNC_NAME, SCM_CAR (list));
-      value = SCM_NUM2INT (SCM_ARGN, SCM_CAR (list));
-      if (value < 0 || value > 255)
-	scm_out_of_range (FUNC_NAME, SCM_CAR (list));
-      *p++ = value;
+      val = SCM_CAR (list);
+      if (!SCM_EXACTP (val) && !SCM_CHARP (val))
+	{
+	  scm_must_free ((void *) bin->data);
+	  scm_must_free ((void *) bin);
+	  scm_wrong_type_arg_msg (FUNC_NAME, SCM_ARGn, val, "char or exact");
+	}
+      value = SCM_CHARP (val) ? 
+	((int) SCM_CHAR (val)) : SCM_NUM2INT (SCM_ARGn, val);
+      if (value < -128 || value > 255)
+	{
+	  scm_must_free ((void *) bin->data);
+	  scm_must_free ((void *) bin);
+	  SCM_OUT_OF_RANGE (SCM_ARGn, val);
+	}
+      *p++ = (unsigned char) value;
       list = SCM_CDR (list);
     }
 
@@ -457,6 +551,22 @@ guile_data_to_bin (void *data, int size)
   SCM_RETURN_NEWSMOB (guile_bin_tag, bin);
 }
 
+/* Converts the data pointer @var{data} with a size of @var{size} bytes
+   into a binary smob which is marked as garbage.  This means the data 
+   pointer must be allocated by @code{scm_must_malloc()} or 
+   @code{scm_must_realloc()}. */
+SCM
+guile_garbage_to_bin (void *data, int size)
+{
+  guile_bin_t *bin;
+     
+  bin = MAKE_BIN_SMOB ();
+  bin->size = size;
+  bin->data = data;
+  bin->garbage = 1;
+  SCM_RETURN_NEWSMOB (guile_bin_tag, bin);
+}
+
 /* This function converts the given binary smob @var{binary} back to a
    data pointer. If the @var{size} argument is not NULL the current size
    of the binary smob will be stored there. */
@@ -473,13 +583,15 @@ guile_bin_to_data (SCM binary, int *size)
    binary smob's data for reading depending on the given @var{ctype}. */
 #define MAKE_BIN_REF(ctype)                                                  \
 static SCM GUILE_CONCAT3 (guile_bin_,ctype,_ref) (SCM binary, SCM index) {   \
-  guile_bin_t *bin; int idx;                                                 \
+  guile_bin_t *bin; int idx; long val = 0; void *data;                       \
   CHECK_BIN_SMOB_ARG (binary, SCM_ARG1, bin);                                \
   SCM_ASSERT_TYPE (SCM_EXACTP (index), index, SCM_ARG2, FUNC_NAME, "exact"); \
   idx = SCM_NUM2INT (SCM_ARG2, index);                                       \
   if (idx < 0 || idx >= (int) (bin->size / sizeof (ctype)))                  \
     SCM_OUT_OF_RANGE (SCM_ARG2, index);                                      \
-  return scm_long2num ((long) ((ctype *) bin->data)[idx]);                   \
+  data = SVZ_NUM2PTR (SVZ_PTR2NUM (bin->data) + idx * sizeof (ctype));       \
+  memcpy (&val, data, sizeof (ctype));                                       \
+  return scm_long2num (val);                                                 \
 }
 
 /* Checks whether the scheme value @var{value} can be stored within a
@@ -504,7 +616,8 @@ static SCM GUILE_CONCAT3 (guile_bin_,ctype,_ref) (SCM binary, SCM index) {   \
 #define MAKE_BIN_SET(ctype)                                                  \
 static SCM GUILE_CONCAT3 (guile_bin_,ctype,_set) (SCM binary, SCM index,     \
 						  SCM value) {               \
-  guile_bin_t *bin; int idx; unsigned long val; SCM old; ctype *data;        \
+  guile_bin_t *bin; int idx; unsigned long val; SCM old;                     \
+  unsigned long oldval = 0; void *data;                                      \
   CHECK_BIN_SMOB_ARG (binary, SCM_ARG1, bin);                                \
   SCM_ASSERT_TYPE (SCM_EXACTP (index), index, SCM_ARG2, FUNC_NAME, "exact"); \
   idx = SCM_NUM2INT (SCM_ARG2, index);                                       \
@@ -512,8 +625,9 @@ static SCM GUILE_CONCAT3 (guile_bin_,ctype,_set) (SCM binary, SCM index,     \
     SCM_OUT_OF_RANGE (SCM_ARG2, index);                                      \
   SCM_ASSERT_TYPE (SCM_EXACTP (value), value, SCM_ARG3, FUNC_NAME, "exact"); \
   CTYPE_CHECK_RANGE (ctype, value, SCM_ARG3, val);                           \
-  data = (ctype *) bin->data;                                                \
-  old = scm_long2num ((long) data[idx]); data[idx] = (ctype) val;            \
+  data = SVZ_NUM2PTR (SVZ_PTR2NUM (bin->data) + idx * sizeof (ctype));       \
+  memcpy (&oldval, data, sizeof (ctype));                                    \
+  old = scm_long2num (oldval); memcpy (data, &val, sizeof (ctype));          \
   return old;                                                                \
 }
 
@@ -599,6 +713,8 @@ guile_bin_init (void)
   scm_c_define_gsubr ("binary-length", 1, 0, 0, guile_bin_length);
   scm_c_define_gsubr ("binary-concat!", 2, 0, 0, guile_bin_concat_x);
   scm_c_define_gsubr ("binary-subset", 2, 1, 0, guile_bin_subset);
+  scm_c_define_gsubr ("binary-reverse", 1, 0, 0, guile_bin_reverse);
+  scm_c_define_gsubr ("binary-reverse!", 1, 0, 0, guile_bin_reverse_x);
 
   scm_c_define_gsubr ("binary-long-ref", 2, 0, 0, guile_bin_long_ref);
   scm_c_define_gsubr ("binary-int-ref", 2, 0, 0, guile_bin_int_ref);
