@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile-server.c,v 1.43 2002/02/10 11:38:28 ela Exp $
+ * $Id: guile-server.c,v 1.51 2002/07/30 22:39:08 ela Exp $
  *
  */
 
@@ -54,7 +54,7 @@ static svz_hash_t *guile_sock = NULL;
 /* List of functions to configure for a server type. */
 static char *guile_functions[] = {
   "global-init", "init", "detect-proto", "connect-socket", "finalize",
-  "global-finalize", "info-client", "info-server", "notify", 
+  "global-finalize", "info-client", "info-server", "notify", "reset",
   "handle-request", NULL };
 
 /* If set to zero exception handling is disabled. */
@@ -96,7 +96,7 @@ static int guile_use_exceptions = 1;
  * g) free    - Run if the SMOB gets destroyed.
  */
 #define MAKE_SMOB_DEFINITION(ctype, description)                             \
-static long GUILE_CONCAT3 (guile_,ctype,_tag) = 0;                           \
+static scm_t_bits GUILE_CONCAT3 (guile_,ctype,_tag) = 0;                     \
 static SCM GUILE_CONCAT3 (guile_,ctype,_create) (void *data) {               \
   SCM_RETURN_NEWSMOB (GUILE_CONCAT3 (guile_,ctype,_tag), data);              \
 }                                                                            \
@@ -460,6 +460,7 @@ guile_call (SCM code, int args, ...)
 }
 
 /* Wrapper function for the global initialization of a server type. */
+#define FUNC_NAME "guile_func_global_init"
 static int
 guile_func_global_init (svz_servertype_t *stype)
 {
@@ -473,8 +474,10 @@ guile_func_global_init (svz_servertype_t *stype)
     }
   return 0;
 }
+#undef FUNC_NAME
 
 /* Wrapper function for the initialization of a server instance. */
+#define FUNC_NAME "guile_func_init"
 static int
 guile_func_init (svz_server_t *server)
 {
@@ -489,8 +492,10 @@ guile_func_init (svz_server_t *server)
     }
   return 0;
 }
+#undef FUNC_NAME
 
 /* Wrapper routine for protocol detection of a server instance. */
+#define FUNC_NAME "guile_func_detect_proto"
 static int
 guile_func_detect_proto (svz_server_t *server, svz_socket_t *sock)
 {
@@ -506,6 +511,7 @@ guile_func_detect_proto (svz_server_t *server, svz_socket_t *sock)
     }
   return 0;
 }
+#undef FUNC_NAME
 
 /* Free the socket boundary if set by guile. */
 static void
@@ -522,6 +528,7 @@ guile_sock_clear_boundary (svz_socket_t *sock)
 /* Wrapper for the socket disconnected callback. Used here in order to
    delete the additional guile callbacks associated with the disconnected
    socket structure. */
+#define FUNC_NAME "guile_func_disconnected_socket"
 static int
 guile_func_disconnected_socket (svz_socket_t *sock)
 {
@@ -549,8 +556,10 @@ guile_func_disconnected_socket (svz_socket_t *sock)
 
   return retval;
 }
+#undef FUNC_NAME
 
 /* Wrapper for the kicked socket callback. */
+#define FUNC_NAME "guile_func_kicked_socket"
 static int
 guile_func_kicked_socket (svz_socket_t *sock, int reason)
 {
@@ -564,8 +573,10 @@ guile_func_kicked_socket (svz_socket_t *sock, int reason)
     }
   return 0;
 }
+#undef FUNC_NAME
 
 /* Wrapper function for the socket connection after successful detection. */
+#define FUNC_NAME "guile_func_connect_socket"
 static int
 guile_func_connect_socket (svz_server_t *server, svz_socket_t *sock)
 {
@@ -584,8 +595,10 @@ guile_func_connect_socket (svz_server_t *server, svz_socket_t *sock)
     }
   return 0;
 }
+#undef FUNC_NAME
 
 /* Wrapper for the finalization of a server instance. */
+#define FUNC_NAME "guile_func_finalize"
 static int
 guile_func_finalize (svz_server_t *server)
 {
@@ -609,8 +622,10 @@ guile_func_finalize (svz_server_t *server)
 
   return retval;
 }
+#undef FUNC_NAME
 
 /* Wrapper routine for the global finalization of a server type. */
+#define FUNC_NAME "guile_func_global_finalize"
 static int
 guile_func_global_finalize (svz_servertype_t *stype)
 {
@@ -624,12 +639,14 @@ guile_func_global_finalize (svz_servertype_t *stype)
     }
   return 0;
 }
+#undef FUNC_NAME
 
 /* Min-Max definitions. */
 #define GUILE_MIN(x, y) (((x) < (y)) ? (x) : (y))
 #define GUILE_MAX(x, y) (((x) > (y)) ? (x) : (y))
 
 /* Wrapper for the client info callback. */
+#define FUNC_NAME "guile_func_info_client"
 static char *
 guile_func_info_client (svz_server_t *server, svz_socket_t *sock)
 {
@@ -653,8 +670,10 @@ guile_func_info_client (svz_server_t *server, svz_socket_t *sock)
     }
   return NULL;
 }
+#undef FUNC_NAME
 
 /* Wrapper for the server info callback. */
+#define FUNC_NAME "guile_func_info_server"
 static char *
 guile_func_info_server (svz_server_t *server)
 {
@@ -677,8 +696,10 @@ guile_func_info_server (svz_server_t *server)
     }
   return NULL;
 }
+#undef FUNC_NAME
 
 /* Wrapper for the server notifier callback. */
+#define FUNC_NAME "guile_func_notify"
 static int
 guile_func_notify (svz_server_t *server)
 {
@@ -692,8 +713,27 @@ guile_func_notify (svz_server_t *server)
     }
   return -1;
 }
+#undef FUNC_NAME
+
+/* Wrapper for the server reset callback. */
+#define FUNC_NAME "guile_func_reset"
+static int
+guile_func_reset (svz_server_t *server)
+{
+  svz_servertype_t *stype = svz_servertype_find (server);
+  SCM ret, reset = guile_servertype_getfunction (stype, "reset");
+
+  if (!SCM_UNBNDP (reset))
+    {
+      ret = guile_call (reset, 1, MAKE_SMOB (svz_server, server));
+      return guile_integer (SCM_ARGn, ret, -1);
+    }
+  return -1;
+}
+#undef FUNC_NAME
 
 /* Wrapper for the socket check request callback. */
+#define FUNC_NAME "guile_func_check_request"
 static int
 guile_func_check_request (svz_socket_t *sock)
 {
@@ -707,9 +747,11 @@ guile_func_check_request (svz_socket_t *sock)
     }
   return -1;
 }
+#undef FUNC_NAME
 
 /* Wrapper for the socket handle request callback. The function searches for
    both the servertype specific and socket specific procedure. */
+#define FUNC_NAME "guile_func_handle_request"
 static int
 guile_func_handle_request (svz_socket_t *sock, char *request, int len)
 {
@@ -733,8 +775,10 @@ guile_func_handle_request (svz_socket_t *sock, char *request, int len)
     }
   return -1;
 }
+#undef FUNC_NAME
 
 /* Wrapper for the socket idle func callback. */
+#define FUNC_NAME "guile_func_idle_func"
 static int
 guile_func_idle_func (svz_socket_t *sock)
 {
@@ -747,6 +791,57 @@ guile_func_idle_func (svz_socket_t *sock)
     }
   return 0;
 }
+#undef FUNC_NAME
+
+/* Wrapper for the socket trigger condition func callback. */
+#define FUNC_NAME "guile_func_trigger_cond"
+static int
+guile_func_trigger_cond (svz_socket_t *sock)
+{
+  SCM ret, trigger_cond = guile_sock_getfunction (sock, "trigger-condition");
+
+  if (!SCM_UNBNDP (trigger_cond))
+    {
+      ret = guile_call (trigger_cond, 1, MAKE_SMOB (svz_socket, sock));
+      return SCM_NFALSEP (ret);
+    }
+  return 0;
+}
+#undef FUNC_NAME
+
+/* Wrapper for the socket trigger func callback. */
+#define FUNC_NAME "guile_func_trigger_func"
+static int
+guile_func_trigger_func (svz_socket_t *sock)
+{
+  SCM ret, trigger_func = guile_sock_getfunction (sock, "trigger");
+
+  if (!SCM_UNBNDP (trigger_func))
+    {
+      ret = guile_call (trigger_func, 1, MAKE_SMOB (svz_socket, sock));
+      return guile_integer (SCM_ARGn, ret, -1);
+    }
+  return 0;
+}
+#undef FUNC_NAME
+
+/* Wrapper for the socket check oob request callback. */
+#define FUNC_NAME "guile_func_check_request_oob"
+static int
+guile_func_check_request_oob (svz_socket_t *sock)
+{
+  SCM ret, check_request_oob;
+  check_request_oob = guile_sock_getfunction (sock, "check-oob-request");
+
+  if (!SCM_UNBNDP (check_request_oob))
+    {
+      ret = guile_call (check_request_oob, 2, MAKE_SMOB (svz_socket, sock),
+			scm_int2num (sock->oob));
+      return guile_integer (SCM_ARGn, ret, -1);
+    }
+  return -1;
+}
+#undef FUNC_NAME
 
 /* Set the @code{handle-request} member of the socket structure @var{sock} 
    to the Guile procedure @var{proc}. The procedure returns the previously
@@ -763,7 +858,7 @@ MAKE_SOCK_CALLBACK (check_request, "check-request")
 #undef FUNC_NAME
 
 /* Setup the packet boundary of the socket @var{sock}. The given string value
-   @var{boundary} can contain any kind of data. If you pass a exact number 
+   @var{boundary} can contain any kind of data. If you pass an exact number 
    value the socket is setup to parse fixed sized packets. In fact this 
    procedure sets the @code{check-request} callback of the given socket 
    structure @var{sock} to a predefined routine which runs the 
@@ -1005,7 +1100,6 @@ guile_server_state_ref (SCM server, SCM key)
   SCM ret = SCM_EOL;
   svz_server_t *xserver;
   char *str;
-  void *val;
   svz_hash_t *hash;
 
   CHECK_SERVER_SMOB_ARG (server, SCM_ARG1, xserver);
@@ -1013,8 +1107,8 @@ guile_server_state_ref (SCM server, SCM key)
   str = guile_to_string (key);
 
   if ((hash = xserver->data) != NULL)
-    if ((val = svz_hash_get (hash, str)) != NULL)
-      ret = (SCM) SVZ_PTR2NUM (val);
+    if (svz_hash_exists (hash, str))
+      ret = (SCM) SVZ_PTR2NUM (svz_hash_get (hash, str));
   scm_c_free (str);
   return ret;
 }
@@ -1032,7 +1126,6 @@ guile_server_state_set_x (SCM server, SCM key, SCM value)
   SCM ret = SCM_EOL;
   svz_server_t *xserver;
   char *str;
-  void *val;
   svz_hash_t *hash;
 
   CHECK_SERVER_SMOB_ARG (server, SCM_ARG1, xserver);
@@ -1044,11 +1137,13 @@ guile_server_state_set_x (SCM server, SCM key, SCM value)
       hash = svz_hash_create (4, (svz_free_func_t) guile_unprotect);
       xserver->data = hash;
     }
-  if ((val = svz_hash_put (hash, str, SVZ_NUM2PTR (value))) != NULL)
+  if (svz_hash_exists (hash, str))
     {
-      ret = (SCM) SVZ_PTR2NUM (val);
+      ret = (SCM) SVZ_PTR2NUM (svz_hash_put (hash, str, SVZ_NUM2PTR (value)));
       scm_gc_unprotect_object (ret);
     }
+  else
+    svz_hash_put (hash, str, SVZ_NUM2PTR (value));
   scm_gc_protect_object (value);
   scm_c_free (str);
   return ret;
@@ -1451,7 +1546,8 @@ guile_servertype_config (svz_servertype_t *server, SCM cfg)
 /*
  * Guile server definition: This procedure takes one argument containing
  * the information about a new server type. If everything works fine you
- * have a freshly registered server type afterwards. Return #t on success.
+ * have a freshly registered server type afterwards. Return @code{#t} on 
+ * success.
  */
 #define FUNC_NAME "define-servertype!"
 SCM
@@ -1495,9 +1591,19 @@ guile_define_servertype (SCM args)
 	scm_gc_protect_object (proc);
     }
 
-  /* Check the configuration items for this servertype. */
-  err |= guile_servertype_config (server, 
-				  optionhash_get (options, "configuration"));
+  /* Check duplicate server types. */
+  if (svz_servertype_get (server->prefix, 0) != NULL)
+    {
+      guile_error ("Duplicate servertype definition: `%s'", server->prefix);
+      err = -1;
+    }
+  else
+    {
+      /* Check the configuration items for this servertype. */
+      err |= guile_servertype_config (server, 
+				      optionhash_get (options, 
+						      "configuration"));
+    }
 
   if (!err)
     {
@@ -1510,11 +1616,15 @@ guile_define_servertype (SCM args)
       server->info_client = guile_func_info_client;
       server->info_server = guile_func_info_server;
       server->notify = guile_func_notify;
+      server->reset = guile_func_reset;
       server->handle_request = guile_func_handle_request;
       guile_servertype_add (server, functions);
     }
   else
     {
+      svz_free (server->prefix);
+      if (server->description)
+	svz_free (server->description);
       svz_free (server);
       svz_hash_destroy (functions);
     }
@@ -1522,6 +1632,7 @@ guile_define_servertype (SCM args)
  out:
   optionhash_destroy (options);
   svz_free (txt);
+  guile_global_error |= err;
   return err ? SCM_BOOL_F : SCM_BOOL_T;
 }
 #undef FUNC_NAME
