@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: guile.c,v 1.59 2002/01/06 13:18:52 ela Exp $
+ * $Id: guile.c,v 1.68 2002/09/28 10:33:48 ela Exp $
  *
  */
 
@@ -32,6 +32,15 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/stat.h>
+
+#if HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+#ifdef __MINGW32__
+# include <io.h>
+#endif
+
 #if GUILE_SOURCE
 # include <libguile/gh.h>
 #else
@@ -47,7 +56,7 @@
  * Global error flag that indicating failure of one of the parsing 
  * functions.
  */
-static int guile_global_error = 0;
+int guile_global_error = 0;
 
 /* 
  * Global variable containing the current load port in exceptions. 
@@ -155,9 +164,11 @@ guile_error (char *format, ...)
     gh_scm2newstr (SCM_FILENAME (lp), NULL) : NULL;
 
   /* guile counts lines from 0, we have to add one */
-  fprintf (stderr, "%s:%d:%d: ", file ? file : "undefined", 
-	   (!SCM_UNBNDP (lp) && SCM_PORTP (lp)) ? SCM_LINUM (lp) + 1 : 0, 
-	   (!SCM_UNBNDP (lp) && SCM_PORTP (lp)) ? SCM_COL (lp) : 0);
+  fprintf (stderr, "%s:%d:%d: ", file ? file : "undefined",
+	   (!SCM_UNBNDP (lp) && SCM_PORTP (lp)) ?
+	   (int) SCM_LINUM (lp) + 1 : 0,
+	   (!SCM_UNBNDP (lp) && SCM_PORTP (lp)) ?
+	   (int) SCM_COL (lp) : 0);
   if (file)
     scm_c_free (file);
 
@@ -232,12 +243,12 @@ optionhash_get (svz_hash_t *hash, char *key)
 
 /*
  * Traverse a scheme pairlist that is an associative list and build up
- * a hash from it. Emits error messages and returns NULL when it did so.
- * Hash keys are the key names. Hash values are pointers to guile_value_t 
- * structures. If @var{dounpack} is set the pairlist's car is used instead
- * of the pairlist itself. You have to unpack when no "." is in front of
- * the pairlist definitiion (in scheme code). Some please explain the "." to
- * Ela and Raimi...
+ * a hash from it. Emits error messages and returns NULL when it did
+ * so.  Hash keys are the key names. Hash values are pointers to
+ * guile_value_t structures. If @var{dounpack} is set the pairlist's
+ * car is used instead of the pairlist itself. You have to unpack when
+ * no "." is in front of the pairlist definition (in scheme
+ * code). Some please explain the "." to Stefan and Raimi...
  */
 svz_hash_t *
 guile_to_optionhash (SCM pairlist, char *txt, int dounpack)
@@ -308,6 +319,7 @@ guile_to_optionhash (SCM pairlist, char *txt, int dounpack)
  * Stores the integer value where @var{target} points to. Does not emit 
  * error messages.
  */
+#define FUNC_NAME "guile_to_integer"
 int
 guile_to_integer (SCM cell, int *target)
 {
@@ -335,6 +347,7 @@ guile_to_integer (SCM cell, int *target)
     }
   return err;
 }
+#undef FUNC_NAME
 
 /*
  * Parse a boolean value from a scheme cell. We consider integers and #t/#f
@@ -342,6 +355,7 @@ guile_to_integer (SCM cell, int *target)
  * zero when successful. Stores the boolean/integer where @var{target} points
  * to. Does not emit error messages.
  */
+#define FUNC_NAME "guile_to_boolean"
 int
 guile_to_boolean (SCM cell, int *target)
 {
@@ -381,6 +395,7 @@ guile_to_boolean (SCM cell, int *target)
     }
   return err;
 }
+#undef FUNC_NAME
 
 /*
  * Convert the given guile list @var{list} into a hash. Return NULL on
@@ -1081,7 +1096,7 @@ guile_define_server (SCM name, SCM args)
   else if (NULL == (options = guile_to_optionhash (args, txt, 0)))
     FAIL (); /* Message already emitted. */
 
-  /* Seperate server description. */
+  /* Separate server description. */
   p = servertype = svz_strdup (servername);
   while (*p && *p != '-')
     p++;
@@ -1117,12 +1132,13 @@ guile_define_server (SCM name, SCM args)
   /* Add server if configuration is ok, no error yet. */
   if (!err)
     {
-      if ((server = svz_server_add (server)) != NULL)
+      if (svz_server_get (servername) != NULL)
 	{
 	  guile_error ("Duplicate server definition: `%s'", servername);
 	  svz_server_free (server);
 	  FAIL ();
 	}
+      svz_server_add (server);
     }
   else
     {
@@ -1364,13 +1380,14 @@ guile_define_port (SCM name, SCM args)
   if (err)
     FAIL ();
 
-  if ((prev = svz_portcfg_add (portname, cfg)) != cfg)
+  if ((prev = svz_portcfg_get (portname)) != NULL)
     {
       /* We've overwritten something. Report and dispose. */
       guile_error ("Duplicate definition of port `%s'", portname);
       err = -1;
-      svz_portcfg_destroy (prev);
     }
+  else
+    svz_portcfg_add (portname, cfg);
 
  out:
   if (err)
@@ -1498,7 +1515,7 @@ guile_hash_to_guile (svz_hash_t *hash)
 }
 
 /*
- * Make the list of local interfaces accessable for guile. Returns the
+ * Make the list of local interfaces accessible for Guile. Returns the
  * local interfaces as a list of ip addresses in dotted decimal form. If
  * another list is given in @var{args} it should contain the new list of
  * local interfaces.
@@ -1551,12 +1568,12 @@ guile_access_interfaces (SCM args)
 #undef FUNC_NAME
 
 /*
- * Make the search path for the Serveez core library accessable for guile.
+ * Make the search path for the Serveez core library accessible for Guile.
  * Returns a list a each path as previously defined. Can override the current
  * definition of this load path. The load path is used to tell Serveez where
  * it can find additional server modules.
  */
-#define FUNC_NAME "serveez-load-path"
+#define FUNC_NAME "serveez-loadpath"
 SCM
 guile_access_loadpath (SCM args)
 {
@@ -1666,42 +1683,6 @@ MAKE_INT_ACCESSOR (guile_access_maxsockets, svz_config.max_sockets)
 MAKE_STRING_ACCESSOR (guile_access_passwd, svz_config.password)
 #undef FUNC_NAME
 
-/*
- * Initialize Guile. Make certain variables and functions defined above
- * available to Guile.
- */
-static void
-guile_init (void)
-{
-  /* define some variables */
-  scm_c_define ("serveez-version", scm_makfrom0str (svz_version));
-  scm_c_define ("guile-version", scm_version ());
-  scm_c_define ("have-debug", SCM_BOOL (svz_have_debug));
-  scm_c_define ("have-Win32", SCM_BOOL (svz_have_Win32));
-  scm_c_define ("have-floodprotect", SCM_BOOL (svz_have_floodprotect));
-
-  /* export accessors for global variables (read/write capable) */
-  scm_c_define_gsubr ("serveez-verbosity", 0, 1, 0, guile_access_verbosity);
-  scm_c_define_gsubr ("serveez-maxsockets", 0, 1, 0, guile_access_maxsockets);
-  scm_c_define_gsubr ("serveez-passwd", 0, 1, 0, guile_access_passwd);
-  scm_c_define_gsubr ("serveez-interfaces", 0, 1, 0, guile_access_interfaces);
-  scm_c_define_gsubr ("serveez-loadpath", 0, 1, 0, guile_access_loadpath);
-
-  /* export some new procedures */
-  scm_c_define_gsubr ("define-port!", 2, 0, 0, guile_define_port);
-  scm_c_define_gsubr ("define-server!", 1, 1, 0, guile_define_server);
-  scm_c_define_gsubr ("bind-server!", 2, 0, 0, guile_bind_server);
-
-  /* export checker functions */
-  scm_c_define_gsubr ("serveez-port?", 1, 0, 0, guile_check_port);
-  scm_c_define_gsubr ("serveez-server?", 1, 0, 0, guile_check_server);
-  scm_c_define_gsubr ("serveez-servertype?", 1, 0, 0, guile_check_stype);
-
-#if ENABLE_GUILE_SERVER
-  guile_server_init ();
-#endif /* ENABLE_GUILE_SERVER */
-}
-
 /*
  * Exception handler for guile. It is called if the evaluation of the file
  * evaluator or a guile procedure call failed.
@@ -1736,11 +1717,127 @@ guile_exception (void *data, SCM tag, SCM args)
   return SCM_BOOL_F;
 }
 
+/* Wrapper for the exception handler in @code{(serveez-load)}. */
+static SCM
+guile_serveez_load_file (void *data)
+{
+  return scm_c_primitive_load ((char *) data);
+}
+
+/*
+ * This procedure can be used as a replacement for @code{(primitive-load)}
+ * in serveez configuration files.  It tries to locate the given filename
+ * @var{file} in the paths returned by @code{(serveez-loadpath)}.  If 
+ * @var{file} cannot be loaded the procedure returns @code{#f}.
+ */
+#define FUNC_NAME "serveez-load"
+static SCM
+guile_serveez_load (SCM file)
+{
+  char *f, *path, *full;
+  int n;
+  SCM ret;
+  svz_array_t *paths = svz_dynload_path_get ();
+
+  GUILE_PRECALL ();
+
+  /* Check argument. */
+  if ((full = guile_to_string (file)) == NULL)
+    return SCM_BOOL_F;
+  f = svz_strdup (full);
+  scm_c_free (full);
+
+  if (svz_file_check (f) == -1)
+    {
+      /* Iterate the loadpath and check if the given file is in there. */
+      svz_array_foreach (paths, path, n)
+	{
+	  full = svz_file_path (path, f);
+	  if (svz_file_check (full) != -1)
+	    {
+	      svz_free (f);
+	      f = full;
+	      break;
+	    }
+	  svz_free (full);
+	}
+    }
+  svz_array_destroy (paths);
+
+  /* Evaluate (load) file and catch exception. */
+  ret = scm_internal_catch (SCM_BOOL_T,
+			    (scm_catch_body_t) guile_serveez_load_file,
+			    (void *) f,
+			    (scm_catch_handler_t) guile_exception,
+			    (void *) f);
+  svz_free (f);
+  return ret;
+}
+#undef FUNC_NAME
+
+/*
+ * Initialize Guile. Make certain variables and functions defined above
+ * available to Guile.
+ */
+static void
+guile_init (void)
+{
+  /* define some variables */
+  scm_c_define ("serveez-version", scm_makfrom0str (svz_version));
+  scm_c_define ("guile-version", scm_version ());
+  scm_c_define ("have-debug", SCM_BOOL (svz_have_debug));
+  scm_c_define ("have-Win32", SCM_BOOL (svz_have_Win32));
+  scm_c_define ("have-floodprotect", SCM_BOOL (svz_have_floodprotect));
+
+  /* export accessors for global variables (read/write capable) */
+  scm_c_define_gsubr ("serveez-verbosity", 0, 1, 0, guile_access_verbosity);
+  scm_c_define_gsubr ("serveez-maxsockets", 0, 1, 0, guile_access_maxsockets);
+  scm_c_define_gsubr ("serveez-passwd", 0, 1, 0, guile_access_passwd);
+  scm_c_define_gsubr ("serveez-interfaces", 0, 1, 0, guile_access_interfaces);
+  scm_c_define_gsubr ("serveez-loadpath", 0, 1, 0, guile_access_loadpath);
+
+  /* export some new procedures */
+  scm_c_define_gsubr ("define-port!", 2, 0, 0, guile_define_port);
+  scm_c_define_gsubr ("define-server!", 1, 1, 0, guile_define_server);
+  scm_c_define_gsubr ("bind-server!", 2, 0, 0, guile_bind_server);
+
+  /* export checker functions */
+  scm_c_define_gsubr ("serveez-port?", 1, 0, 0, guile_check_port);
+  scm_c_define_gsubr ("serveez-server?", 1, 0, 0, guile_check_server);
+  scm_c_define_gsubr ("serveez-servertype?", 1, 0, 0, guile_check_stype);
+
+  /* primitive load including the loadpath */
+  scm_c_define_gsubr ("serveez-load", 1, 0, 0, guile_serveez_load);
+
+#if ENABLE_GUILE_SERVER
+  guile_server_init ();
+#endif /* ENABLE_GUILE_SERVER */
+}
+
 /* Wrapper function for the file loader exception handler. */
 static SCM
 guile_eval_file (void *data)
 {
   char *file = (char *) data;
+  struct stat buf;
+  int error;
+
+  /* Parse configuration from standard input stream. */
+#ifdef __MINGW32__
+  error = svz_fstat ((int) GetStdHandle (STD_INPUT_HANDLE), &buf);
+#else
+  error = svz_fstat (fileno (stdin), &buf);
+#endif
+  if (file == NULL || (error != -1 && !isatty (fileno (stdin)) &&
+		       !S_ISCHR (buf.st_mode) && !S_ISBLK (buf.st_mode)))
+    {
+      SCM ret = SCM_BOOL_F, line;
+      while (!SCM_EOF_OBJECT_P (line = scm_read (scm_def_inp)))
+	ret = scm_primitive_eval_x (line);
+      return SCM_BOOL_T;
+    }
+
+  /* Load configuration from file. */
   return scm_c_primitive_load (file);
 }
 
