@@ -1,7 +1,7 @@
 /*
  * core.c - socket and file descriptor core implementations
  *
- * Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: core.c,v 1.26 2002/01/20 17:12:29 ela Exp $
+ * $Id: core.c,v 1.28 2002/07/28 12:13:17 ela Exp $
  *
  */
 
@@ -162,7 +162,7 @@ svz_fd_cloexec (int fd)
 }
 
 /*
- * This funtion creates an unnamed pair of connected sockets with the 
+ * This function creates an unnamed pair of connected sockets with the 
  * specified protocol @var{proto}. The descriptors used in referencing the 
  * new sockets are returned in desc[0] and desc[1]. The two sockets are 
  * indistinguishable. Also make both of them non-blocking and 
@@ -545,7 +545,7 @@ svz_sendfile (int out_fd, int in_fd, off_t *offset, unsigned int count)
 
   /* TransmitFile().
      This system call provides something likely the Unix sendfile(). It
-     is a M$ specific entension to Winsock. The function comes from the
+     is a M$ specific extension to Winsock. The function comes from the
      MSWSOCK.DLL and should be prototyped in MSWSOCK.H. It operates on
      file handles gained from kernel32.CreateFile() only. We experienced
      quite low performance on small (less than 4096 byte) file chunks. 
@@ -603,9 +603,15 @@ svz_sendfile (int out_fd, int in_fd, off_t *offset, unsigned int count)
       ret = count;
     }
 
+#elif defined (__hpux)
+  
+  /* HP-UX 11i */
+  ret = sendfile (out_fd, in_fd, *offset, (bsize_t) count, NULL, 0);
+  *offset += ((ret >= 0) ? ret : 0);
+
 #else 
 
-  /* Linux here. Works like charme... */
+  /* Linux here. Works like charm... */
   ret = sendfile (out_fd, in_fd, offset, count);
 
 #endif
@@ -859,3 +865,32 @@ svz_fclose (FILE *f)
     }
   return 0;
 }
+
+/*
+ * Checks for the existence of the given file system node @var{file} and 
+ * return zero on success.  Otherwise the function returns non-zero.
+ */
+int
+svz_file_check (char *file)
+{
+  struct stat buf;
+  return file ? stat (file, &buf) : -1;
+}
+
+/*
+ * Constructs a fully qualified file name form @var{path} and @var{file}.
+ * If @var{path} is omitted (@code{NULL}) the function returns @var{file}
+ * only.  If @var{file} is @code{NULL} a null pointer is returned.
+ * Please remember to @code{svz_free()} the returned pointer.
+ */
+char *
+svz_file_path (char *path, char *file)
+{
+  char *full;
+
+  if (file == NULL)
+    return NULL;
+  full = svz_malloc ((path ? strlen (path) + 1 : 0) + strlen (file) + 1);
+  sprintf (full, "%s%s%s", path ? path : "", path ? "/" : "", file);
+  return full;
+}
