@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: coserver.c,v 1.26 2002/02/15 12:16:07 ela Exp $
+ * $Id: coserver.c,v 1.29 2002/05/15 14:00:46 ela Exp $
  *
  */
 
@@ -625,14 +625,16 @@ svz_coserver_close_pipes (svz_coserver_t *self)
 static void
 svz_coserver_closeall (svz_socket_t *self)
 {
-  svz_socket_t *sock;
+  svz_socket_t *sock, *next;
 
-  svz_sock_foreach (sock)
+  for (sock = svz_sock_root; sock != NULL; sock = next)
     {
       if (sock->flags & SOCK_FLAG_SOCK)
-	close (sock->sock_desc);
+	if (sock->sock_desc >= 2)
+	  close (sock->sock_desc);
       if (sock->flags & SOCK_FLAG_FILE)
-	close (sock->file_desc);
+	if (sock->file_desc >= 2)
+	  close (sock->file_desc);
       if (sock->flags & SOCK_FLAG_PIPE)
 	{
 	  if (sock->pipe_desc[READ] >= 2)
@@ -640,6 +642,7 @@ svz_coserver_closeall (svz_socket_t *self)
 	  if (sock->pipe_desc[WRITE] >= 2)
 	    close (sock->pipe_desc[WRITE]);
 	}
+      next = sock->next;
       if (sock != self)
 	{
 	  svz_sock_resize_buffers (sock, 0, 0);
