@@ -1,7 +1,7 @@
 /*
  * dynload.c - dynamic server loading implementation
  *
- * Copyright (C) 2001 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2001, 2002 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dynload.c,v 1.20 2001/12/22 10:32:52 ela Exp $
+ * $Id: dynload.c,v 1.22 2002/07/28 12:13:17 ela Exp $
  *
  */
 
@@ -57,10 +57,12 @@
 # include <winsock2.h>
 #endif
 
+#include "svzpath.h"
 #include "libserveez/alloc.h"
 #include "libserveez/util.h"
 #include "libserveez/socket.h"
 #include "libserveez/server.h"
+#include "libserveez/core.h"
 #include "libserveez/dynload.h"
 
 /* Internal list of shared libraries. */
@@ -99,6 +101,11 @@ dyn_error (void)
 # define DYNLOAD_SUFFIX "dylib"
 # define DYNLOAD_PATH_SEPERATOR ':'
 # define DYNLOAD_SYMBOL_PREFIX "_"
+#elif defined (__hpux)
+# define DYNLOAD_PREFIX "lib"
+# define DYNLOAD_SUFFIX "sl"
+# define DYNLOAD_PATH_SEPERATOR ':'
+# define DYNLOAD_SYMBOL_PREFIX ""
 #else
 # define DYNLOAD_PREFIX "lib"
 # define DYNLOAD_SUFFIX "so"
@@ -138,8 +145,7 @@ dyn_get_library (char *path, char *file)
   void *handle = NULL;
   char *lib;
 
-  lib = svz_malloc ((path ? strlen (path) + 1 : 0) + strlen (file) + 1);
-  sprintf (lib, "%s%s%s", path ? path : "", path ? "/" : "", file);
+  lib = svz_file_path (path, file);
 
 #if HAVE_DLOPEN
   handle = dlopen (lib, RTLD_NOW | RTLD_GLOBAL);
@@ -152,7 +158,7 @@ dyn_get_library (char *path, char *file)
 #elif defined (__MINGW32__)
   handle = LoadLibrary (lib);
 #elif HAVE_SHL_LOAD
-  handle = shl_load (lib, BIND_IMMEDIATE | BIND_NONFATAL | DYNAMIC_PATH);
+  handle = shl_load (lib, BIND_IMMEDIATE | BIND_NONFATAL | DYNAMIC_PATH, 0L);
 #elif HAVE_NSADDIMAGE
   handle = (void *) NSAddImage (lib, NSADDIMAGE_OPTION_RETURN_ON_ERROR |
 				NSADDIMAGE_OPTION_WITH_SEARCHING);
@@ -210,7 +216,13 @@ svz_dynload_path_get (void)
   char *path, *p, *start, *val;
   int len, n;
 
+  /* Add some default paths. */
   svz_array_add (paths, svz_strdup ("."));
+  svz_array_add (paths, svz_strdup (SVZ_LIBDIR));
+  svz_array_add (paths, svz_strdup (SVZ_BINDIR));
+  svz_array_add (paths, svz_strdup (SVZ_PKGDATADIR));
+
+  /* Add environment variable paths. */
   if ((p = getenv (DYNLOAD_PATH)) != NULL)
     {
       while (*p)
