@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: passthrough.c,v 1.16 2001/11/30 16:10:23 ela Exp $
+ * $Id: passthrough.c,v 1.19 2001/12/19 23:15:06 ela Exp $
  *
  */
 
@@ -50,10 +50,6 @@
 # include <sys/wait.h>
 #endif
 
-#if !defined(__MINGW32__) && !defined(__CYGWIN__)
-extern char ** environ;
-#endif
-
 #ifdef __MINGW32__
 # include <winsock2.h>
 # include <io.h>
@@ -72,6 +68,13 @@ extern char ** environ;
 #include "libserveez/pipe-socket.h"
 #include "libserveez/passthrough.h"
 
+/*
+ * This variable is meant to hold the @code{environ} variable of the 
+ * application using the Serveez core API. It must be setup via the macro
+ * @code{svz_envblock_setup()}.
+ */
+char **svz_environ = NULL;
+
 /*
  * This routine starts a new program specified by @var{bin} passing the
  * socket or pipe descriptor(s) in the socket structure @var{sock} to its
@@ -632,22 +635,33 @@ svz_process_create_child (svz_process_t *proc)
      and pass them to the child program. */
   if (proc->flag == SVZ_PROCESS_FORK)
     {
+      HANDLE fd;
+
       if (proc->in != proc->out)
 	{
-	  proc->in = svz_process_duplicate (proc->in, proc->sock->proto);
-	  if (proc->in == INVALID_HANDLE)
+	  /* Create an inheritable receive pipe and replace it. */
+	  fd = svz_process_duplicate (proc->in, proc->sock->proto);
+	  if (fd == INVALID_HANDLE)
 	    return -1;
-	  proc->out = svz_process_duplicate (proc->out, proc->sock->proto);
-	  if (proc->out == INVALID_HANDLE)
+          closehandle (proc->sock->pipe_desc[READ]);
+          proc->in = proc->sock->pipe_desc[READ] = fd;
+
+	  /* Create an inheritable send pipe and replace it. */
+	  fd = svz_process_duplicate (proc->out, proc->sock->proto);
+	  if (fd == INVALID_HANDLE)
 	    return -1;
+          closehandle (proc->sock->pipe_desc[WRITE]);
+          proc->out = proc->sock->pipe_desc[WRITE] = fd;
 	}
       else
 	{
-	  HANDLE fd;
+	  /* Create an inheritable socket and replace it. */
 	  fd = svz_process_duplicate (proc->in, proc->sock->proto);
 	  if (fd == INVALID_HANDLE)
 	    return -1;
 	  proc->in = proc->out = fd;
+          closesocket (proc->sock->sock_desc);
+          proc->sock->sock_desc = (SOCKET) fd;
 	}
     }
 
@@ -675,12 +689,17 @@ svz_process_create_child (svz_process_t *proc)
       return -1;
     }
 
-  /* Create sane environment. */
+  /* Create sane environment and pass the receive and send handle to
+     the child process via two environment variables. */
   if (proc->envp == NULL)
     {
       proc->envp = svz_envblock_create ();
       svz_envblock_default (proc->envp);
     }
+  svz_envblock_add (proc->envp, "%s=%ld", 
+		    SVZ_PROCESS_RECV_HANDLE, (long) proc->in);
+  svz_envblock_add (proc->envp, "%s=%ld", 
+		    SVZ_PROCESS_SEND_HANDLE, (long) proc->out);
 
   /* Concatenate application name. */
   if (proc->app != NULL)
@@ -1088,12 +1107,12 @@ svz_envblock_default (svz_envblock_t *env)
   if (env->size)
     svz_envblock_free (env);
 
-  for (n = 0; environ[n] != NULL; n++)
+  for (n = 0; svz_environ != NULL && svz_environ[n] != NULL; n++)
     {
       env->size++;
       env->entry = svz_realloc (env->entry, 
 				sizeof (char *) * (env->size + 1));
-      env->entry[env->size - 1] = svz_strdup (environ[n]);
+      env->entry[env->size - 1] = svz_strdup (svz_environ[n]);
     }
 
   env->entry[env->size] = NULL;
