@@ -20,7 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: server-core.c,v 1.37 2002/02/15 12:16:07 ela Exp $
+ * $Id: server-core.c,v 1.40 2002/07/25 13:45:37 ela Exp $
  *
  */
 
@@ -98,7 +98,7 @@
  * When @var{svz_nuke_happened} is set to a non-zero value, the server
  * will terminate its main loop.
  */
-int svz_nuke_happened;
+int svz_nuke_happened = 0;
 
 /*
  * When @var{svz_reset_happened} gets set to a non-zero value, the server
@@ -887,7 +887,7 @@ svz_sock_unique_id (svz_socket_t *sock)
 static int
 svz_reset (void)
 {
-  /* FIXME: Maybe `server_t' reset callback ? */
+  svz_server_reset ();
   svz_interface_check ();
   return 0;
 }
@@ -1096,6 +1096,9 @@ svz_signal_up (void)
 #ifdef SIGPIPE
   signal (SIGPIPE, svz_signal_handler);
 #endif
+#ifdef SIGURG
+  signal (SIGURG, svz_signal_handler);
+#endif
 #ifdef SIGSEGV
   signal (SIGSEGV, svz_segfault_exception);
 #endif
@@ -1128,6 +1131,9 @@ svz_signal_dn (void)
 #ifdef SIGPIPE
   signal (SIGPIPE, SIG_DFL);
 #endif
+#ifdef SIGURG
+  signal (SIGURG, SIG_DFL);
+#endif
 #ifdef SIGSEGV
   signal (SIGSEGV, SIG_DFL);
 #endif
@@ -1275,7 +1281,6 @@ svz_loop_pre (void)
    * Setting up control variables. These get set either in the signal 
    * handler or from a command processing routine.
    */
-  svz_nuke_happened = 0;
   svz_reset_happened = 0;
   svz_child_died = 0;
   svz_pipe_broke = 0;
