@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.  
  *
- * $Id: socket.h,v 1.18 2002/02/03 09:34:05 ela Exp $
+ * $Id: socket.h,v 1.20 2002/06/06 20:04:51 ela Exp $
  *
  */
 
@@ -131,6 +131,13 @@ struct svz_socket
    */
   int (* read_socket) (svz_socket_t *sock);
 
+  /*
+   * READ_SOCKET_OOB is run when urgent data has been detected on the 
+   * socket. By default it reads a single byte, stores it in OOB and passes
+   * it to the CHECK_REQUEST_OOB callback.
+   */
+  int (* read_socket_oob) (svz_socket_t *sock);
+
   /*
    * WRITE_SOCKET is called when data is is valid in the output buffer
    * and the socket gets available for writing.  Normally, this simply
@@ -163,6 +170,12 @@ struct svz_socket
    */
   int (* check_request) (svz_socket_t *sock);
 
+  /*
+   * CHECK_REQUEST_OOB is called when urgent data has been read from the 
+   * socket. The received byte is stored in OOB.
+   */
+  int (* check_request_oob) (svz_socket_t *sock);
+
   /*
    * HANDLE_REQUEST gets called when the CHECK_REQUEST got
    * a valid packet.
@@ -175,6 +188,17 @@ struct svz_socket
    */
   int (* child_died) (svz_socket_t *sock);
 
+  /*
+   * Called if TRIGGER_COND returned non-zero.
+   */
+  int (* trigger_func) (svz_socket_t *sock);
+
+  /*
+   * TRIGGER_COND is called once every server loop.  If it returns 
+   * non-zero TRIGGER_FUNC is run.
+   */
+  int (* trigger_cond) (svz_socket_t *sock);
+
   /*
    * IDLE_FUNC gets called from the periodic task scheduler.  Whenever
    * IDLE_COUNTER (see below) is non-zero, it is decremented and
@@ -194,6 +218,10 @@ struct svz_socket
   int flood_limit;		/* Limit of the above before kicking. */
 #endif
 
+  /* Out-of-band data for TCP protocol. This byte is used for both, 
+     receiving and sending. */
+  unsigned char oob;
+
   /* Set to non-zero @code{time()} value if the the socket is temporarily
      unavailable (EAGAIN). This is why we use O_NONBLOCK socket descriptors. */
   int unavailable;              
