@@ -20,7 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: util.h,v 1.19 2001/12/13 18:00:01 ela Exp $
+ * $Id: util.h,v 1.21 2002/07/15 07:45:05 ela Exp $
  *
  */
 
@@ -93,6 +93,15 @@ SERVEEZ_API char *svz_hstrerror __PARAMS ((void));
   ((unsigned char) *p | ((unsigned char) *(p + 1) << 8) | \
   ((unsigned char) *(p + 2) << 16) | ((signed char) *(p + 3) << 24))
 
+/*
+ * Convert the byte array pointed to by @var{p} to a signed 64 bit integer.
+ */
+#define SVZ_INT64(p) \
+  ((unsigned char) *p | ((unsigned char) *(p + 1) << 8) | \
+  ((unsigned char) *(p + 2) << 16) | ((unsigned char) *(p + 3) << 24) \
+  ((unsigned char) *(p + 2) << 32) | ((unsigned char) *(p + 3) << 40) \
+  ((unsigned char) *(p + 2) << 48) | ((signed char) *(p + 3) << 54))
+
 /*
  * Convert the byte array pointed to by @var{p} to a signed 16 bit integer.
  */
@@ -106,6 +115,15 @@ SERVEEZ_API char *svz_hstrerror __PARAMS ((void));
   ((unsigned char) *p | ((unsigned char) *(p + 1) << 8) | \
   ((unsigned char) *(p + 2) << 16) | ((unsigned char) *(p + 3) << 24))
 
+/*
+ * Convert the byte array pointed to by @var{p} to an unsigned 64 bit integer.
+ */
+#define SVZ_UINT64(p) \
+  ((unsigned char) *p | ((unsigned char) *(p + 1) << 8) | \
+  ((unsigned char) *(p + 2) << 16) | ((unsigned char) *(p + 3) << 24) \
+  ((unsigned char) *(p + 2) << 32) | ((unsigned char) *(p + 3) << 40) \
+  ((unsigned char) *(p + 2) << 48) | ((unsigned char) *(p + 3) << 54))
+
 /*
  * Convert the byte array pointed to by @var{p} to an unsigned 16 bit integer.
  */
@@ -185,9 +203,13 @@ __END_DECLS
  * but not within the native Win32 headers.
  */
 #ifndef S_ISDIR
-# define S_ISDIR(Mode) ((Mode) & S_IFDIR)
-# define S_ISCHR(Mode) ((Mode) & S_IFCHR)
-# define S_ISREG(Mode) ((Mode) & S_IFREG)
+# ifndef S_IFBLK
+#  define S_IFBLK 0x3000
+# endif
+# define S_ISDIR(Mode) (((Mode) & S_IFMT) == S_IFDIR)
+# define S_ISCHR(Mode) (((Mode) & S_IFMT) == S_IFCHR)
+# define S_ISREG(Mode) (((Mode) & S_IFMT) == S_IFREG)
+# define S_ISBLK(Mode) (((Mode) & S_IFMT) == S_IFBLK)
 #endif /* not S_ISDIR */
 
 #endif /* __MINGW32__ */
